Attribute VB_Name = "Life"
'/*
'Game of Life
'Life.bas
'Copyright  (C) 1998-2000 Hugues Johnson
'
'This program is free software; you can redistribute it and/or
'modify it under the terms of the GNU General Public License
'as published by the Free Software Foundation; either version 2
'of the License, or (at your option) any later version.
'
'This program is distributed in the hope that it will be useful,
'but WITHOUT ANY WARRANTY; without even the implied warranty of
'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
'the GNU General Public License for more details.
'
'You should have received a copy of the GNU General Public License
'along with this program; if not, write to the Free Software
'Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
'*/

'/* grid size */
Private GRID_SIZE_X As Integer
Private GRID_SIZE_Y As Integer
'/* boolean arrays to hold current & next generations */
Private currentGeneration() As Boolean
Private nextGeneration() As Boolean
'/* probability of a space being initially occupied*/
Private percolation As Single
'/* if the game is running or paused */
Public gameRunning As Boolean
'/* counter for number of generations elapsed */
Public elapsed As Integer
'/* number of lives */
Public currentAlive As Integer
Public Sub startNewGame(currentGen As PictureBox, nextGen As PictureBox, percolationProbability As Single)
    Dim x As Integer
    Dim y As Integer

    Randomize timer
    '/* pause the simulation */
    gameRunning = False
    '/* ensure that percolationProbability is between 0.1 and 1 */
    If (percolationProbability <= 0) Then
        percolation = 0.1
    ElseIf (percolationProbability > 1) Then
        percolation = 1
    Else
        percolation = percolationProbability
    End If
    '/* clear the picture boxes */
    currentGen.Cls
    nextGen.Cls
    '/* randomly populate the current generation based on percolationProbability */
    For x = 0 To GRID_SIZE_X - 1
        For y = 0 To GRID_SIZE_Y - 1
            If (Rnd() < percolation) Then
                currentGeneration(x, y) = True
                nextGeneration(x, y) = True
            Else
                currentGeneration(x, y) = False
                nextGeneration(x, y) = False
            End If
        Next y
    Next x
    '/* determine the next generation */
    determineNextGeneration
    '/* unpause the simulation */
    gameRunning = True

End Sub


Public Sub drawGenerations(currentGen As PictureBox, nextGen As PictureBox, alive As IPictureDisp)
    Dim x As Integer
    Dim y As Integer
    Dim xDimension As Integer
    Dim yDimension As Integer

    '/* determine dimensions */
    xDimension = CInt(currentGen.ScaleWidth) / GRID_SIZE_X
    yDimension = CInt(currentGen.ScaleHeight) / GRID_SIZE_Y
    '/* clear the picture boxes */
    currentGen.Cls
    nextGen.Cls
    '/* reset the current alive counter */
    currentAlive = 0
    '/* draw the generations */
    For x = 0 To GRID_SIZE_X - 1
        For y = 0 To GRID_SIZE_Y - 1
            If (currentGeneration(x, y)) Then
                currentGen.PaintPicture alive, x * xDimension, y * yDimension, xDimension, yDimension
                currentAlive = currentAlive + 1
            End If
            If (nextGeneration(x, y)) Then
                nextGen.PaintPicture alive, x * xDimension, y * yDimension, xDimension, yDimension
            End If
        Next y
    Next x


End Sub

Public Sub determineNextGeneration()
    Dim x As Integer
    Dim y As Integer
    Dim neighbors As Integer
    
    '/* copy the next generation into current generation */
    For x = 0 To GRID_SIZE_X - 1
        For y = 0 To GRID_SIZE_Y - 1
            currentGeneration(x, y) = nextGeneration(x, y)
        Next y
    Next x
    '/* scroll through each element in currentGeneration and count the neighbors */
    For x = 0 To GRID_SIZE_X - 1
        For y = 0 To GRID_SIZE_Y - 1
            '/* set to false by default */
            nextGeneration(x, y) = False
            '/* reset neighbor counter */
            neighbors = 0
            If (x = 0) Then
                If (y = 0) Then
                    If (currentGeneration(x + 1, y + 1)) Then neighbors = neighbors + 1
                    If (currentGeneration(x, y + 1)) Then neighbors = neighbors + 1
                    If (currentGeneration(x + 1, y)) Then neighbors = neighbors + 1
                ElseIf (w = GRID_SIZE_Y - 1) Then
                    If (currentGeneration(x + 1, y - 1)) Then neighbors = neighbors + 1
                    If (currentGeneration(x, y - 1)) Then neighbors = neighbors + 1
                    If (currentGeneration(x + 1, y)) Then neighbors = neighbors + 1
                Else
                    If (currentGeneration(x, y + 1)) Then neighbors = neighbors + 1
                    If (currentGeneration(x, y - 1)) Then neighbors = neighbors + 1
                    If (currentGeneration(x + 1, y + 1)) Then neighbors = neighbors + 1
                    If (currentGeneration(x + 1, y)) Then neighbors = neighbors + 1
                End If
            ElseIf (x = GRID_SIZE_X - 1) Then
                If (y = 0) Then
                    If (currentGeneration(x - 1, y + 1)) Then neighbors = neighbors + 1
                    If (currentGeneration(x, y + 1)) Then neighbors = neighbors + 1
                    If (currentGeneration(x - 1, y)) Then neighbors = neighbors + 1
                ElseIf (y = GRID_SIZE_Y - 1) Then
                    If (currentGeneration(x - 1, y - 1)) Then neighbors = neighbors + 1
                    If (currentGeneration(x, y - 1)) Then neighbors = neighbors + 1
                    If (currentGeneration(x - 1, y)) Then neighbors = neighbors + 1
                Else
                    If (currentGeneration(x, y + 1)) Then neighbors = neighbors + 1
                    If (currentGeneration(x, y - 1)) Then neighbors = neighbors + 1
                    If (currentGeneration(x - 1, y + 1)) Then neighbors = neighbors + 1
                    If (currentGeneration(x - 1, y)) Then neighbors = neighbors + 1
                End If
            Else
                If (y = 0) Then
                    If (currentGeneration(x, y + 1)) Then neighbors = neighbors + 1
                    If (currentGeneration(x - 1, y + 1)) Then neighbors = neighbors + 1
                    If (currentGeneration(x - 1, y)) Then neighbors = neighbors + 1
                    If (currentGeneration(x + 1, y + 1)) Then neighbors = neighbors + 1
                    If (currentGeneration(x + 1, y)) Then neighbors = neighbors + 1
                ElseIf (y = GRID_SIZE_Y - 1) Then
                    If (currentGeneration(x, y - 1)) Then neighbors = neighbors + 1
                    If (currentGeneration(x - 1, y - 1)) Then neighbors = neighbors + 1
                    If (currentGeneration(x - 1, y)) Then neighbors = neighbors + 1
                    If (currentGeneration(x + 1, y - 1)) Then neighbors = neighbors + 1
                    If (currentGeneration(x + 1, y)) Then neighbors = neighbors + 1
                Else
                    If (currentGeneration(x, y + 1)) Then neighbors = neighbors + 1
                    If (currentGeneration(x, y - 1)) Then neighbors = neighbors + 1
                    If (currentGeneration(x - 1, y + 1)) Then neighbors = neighbors + 1
                    If (currentGeneration(x - 1, y)) Then neighbors = neighbors + 1
                    If (currentGeneration(x - 1, y - 1)) Then neighbors = neighbors + 1
                    If (currentGeneration(x + 1, y + 1)) Then neighbors = neighbors + 1
                    If (currentGeneration(x + 1, y - 1)) Then neighbors = neighbors + 1
                    If (currentGeneration(x + 1, y)) Then neighbors = neighbors + 1
                End If
            End If
            '/* if a cell is alive and has 2 or 3 neighbors then next generation stays alive */
            If (currentGeneration(x, y)) Then
                If ((neighbors = 2) Or (neighbors = 3)) Then
                    nextGeneration(x, y) = True
                End If
            Else
                '/* if an empty square has 3 neighbors a new life appears */
                If (neighbors = 3) Then
                    nextGeneration(x, y) = True
                End If
            End If
        Next y
    Next x

End Sub

Public Sub changeGridSize(newX As Integer, newY As Integer)
    
    '/* pause simulation */
    gameRunning = False
   '/* set grid size */
    GRID_SIZE_X = newX
    GRID_SIZE_Y = newY
    '/* redimension arrays */
    ReDim currentGeneration(GRID_SIZE_X, GRID_SIZE_Y)
    ReDim nextGeneration(GRID_SIZE_X, GRID_SIZE_Y)
    '/* unpause simulation */
    gameRunning = True

End Sub
